<?php



require_once 'config.php';
require_once 'Classes/Astro/MJD.php';
require_once 'Classes/Queries/CalQuery.php';
require_once 'Classes/Queries/DateRangeQuery.php';
require_once 'Classes/Model/ObservationHeader.php';
require_once GREEN_ZONE_DIR . 'Utils/GZ_Exception.php';



/**
 * @class ConeQuery
 *
 * This class is an algorithm for querying the observation table for all the observations
 * that are within search_radius of the position given by rajd and decjd in the ConeQueryParameters.
 * The search is further refined by the the filtering options such as project_id, date range etc.
 * After we have retreived all the observations
 **/
class ConeQuery
{
  // PUBLIC

  public function __construct()
  {
    $this->parameters = '';
    $this->sql = '';
    $this->cone_string = '';
    $this->observations = null;
    $this->database = null;
    $this->num_rows_truncated = 0;
  }
  
  public function SetParameters( $s_parameters ) { $this->parameters = $s_parameters; }
  
  public function SetDatabase( $s_database ) { $this->database = $s_database; }
  
  public function GetNumRowsTruncated() { return $this->num_rows_truncated; }

  public function Execute()
  {
    if( $this->parameters == '' )
      throw new GZ_Exception( 'attempted to perform query without parameters' );

    if( $this->database == null )
      throw new GZ_Exception( 'Database not set', 'ConeQuery::Execute' );

    $this->observations = Array();

    $this->BuildQueryString();
    $this->database->DoQuery( $this->sql );
    $this->ExtractResultSet( $this->database->GetLastResult() );
    
    // if we have MAX_ROWS results, see how many were truncated
    if( count( $this->observations ) == MAX_ROWS )
    {
      $this->sql = ' SELECT count(*) as trunc_count ';
      $this->BuildFromString();
      $this->BuildWhereString();
      $this->database->DoQuery( $this->sql );
      $only_row = $this->database->FetchRowAsArray();
      $this->num_rows_truncated = $only_row['trunc_count'] - MAX_ROWS;
    }
  }
  
  public function GetObservations(){ return $this->observations; }
  
  // PRIVATE
      
  private $parameters;
  private $results;
  private $sql;
  private $cone_string;
  private $database;
  private $num_rows_truncated;
  
    //! Convert the result set array to an array of observations
  private function ExtractResultSet( $result_set )
  {
    foreach( $result_set as $next_row )
    {
      $new_obs = new ObservationHeader();
      $new_obs->FromRecord( $next_row );
      $this->observations[] = $new_obs;
    }
  }
  
  private function BuildConeString()
  {
    $psr_rajd = $this->parameters->GetCoordinates()->GetRA()->ToDegrees();
    $psr_decjd = $this->parameters->GetCoordinates()->GetDEC()->ToDegrees();
    
    $this->cone_string = "degrees( acos(cos(radians(rajd-$psr_rajd)) *
					cos(radians(decjd)) *
					cos(radians($psr_decjd)) +
					sin(radians(decjd)) *
					sin(radians($psr_decjd))
                                        )
                                 )";
  }
  
  private function BuildSelectString()
  {
    $this->sql .= ' SELECT *,' . $this->cone_string . ' AS distance ';
  }
  
  private function BuildFromString()
  {
    $this->sql .= ' FROM  observations ';
  }
  
  private function BuildWhereString()
  {
    $this->sql .= ' WHERE 1=1 ';

    $radius = $this->parameters->GetSearchRadius();
    
    if( $radius == '' )
      $radius = 1;
    
    $this->sql .= " AND ( (($this->cone_string - 0.25*(bmaj+bmin)) < ($radius/60.)) ) ";
    
    $project_id = $this->parameters->GetProjectID();
    if( $project_id != '')
      $this->sql .= " AND project_id = '$project_id' ";
    
    $date_range = $this->parameters->GetDateRange();
    $from_mjd = new MJD();
    $from_mjd->FromDate( $date_range->GetFromDate() );
    $to_mjd = new MJD();
    $to_mjd->FromDate( $date_range->GetToDate() );
    $this->sql .= ' AND ( MJD <= ' . ceil( $to_mjd->ToString() );
    $this->sql .= ' OR ';
    $this->BuildAuthorizedProjectsString();
    $this->sql .= ' ) ';
    
    if( $this->parameters->GetDataType() != '' )
      $this->sql .= ' AND data_type = "' . $this->parameters->GetDataType() . '" ';
  }
  
  private function BuildLimitString()
  {
    $this->sql .= ' LIMIT ' . MAX_ROWS;
  }
  
  private function BuildOrderString()
  {
    $this->sql .= ' ORDER BY ' . $this->parameters->GetOrderColumn() .
	          ' ' . $this->parameters->GetOrderDirection();
  }
  
  private function BuildQueryString()
  {
    $this->BuildConeString();
    $this->BuildSelectString();
    $this->BuildFromString();
    $this->BuildWhereString();
    $this->BuildOrderString();
    $this->BuildLimitString();
    
//     echo 'the query built was ' . $this->sql .' <br> ';
  }
  
  private function BuildAuthorizedProjectsString()
  {
    $aps = $this->parameters->GetAuthorizedProjects();
    $naps = count( $aps );
    
    if( $naps == 0 )
      $this->sql .= ' 1=0 ';
    else
    {
      $this->sql .= ' project_id IN (';
      for( $ai = 0; $ai < $naps; $ai ++ )
      {
        $this->sql .= "'" . $authorized_projects[$ai] . "'";
        if( $ai < $naps - 1 )
          $this->sql .= ', ';
      }
      $this->sql .= ' ) ';
    }
  }
};




?>